import pygame
import random
import sys
import time
from pygame import mixer

pygame.init()
mixer.init()


# set up
WIDTH = 400
HEIGHT = 500
screen = pygame.display.set_mode([WIDTH, HEIGHT])
pygame.display.set_caption('2048')
timer = pygame.time.Clock()
fps = 60
font = pygame.font.Font('freesansbold.ttf', 24)
big_letter = pygame.font.Font('freesansbold.ttf',37)

# colors of the blocks
colors = {0: (204, 192, 179),
          2: (238, 228, 218),
          4: (237, 224, 200),
          8: (242, 177, 121),
          16: (245, 149, 99),
          32: (246, 124, 95),
          64: (246, 94, 59),
          128: (237, 207, 114),
          256: (237, 204, 97),
          512: (237, 200, 80),
          1024: (237, 197, 63),
          2048: (237, 194, 46),
          'light text': (249, 246, 242),
          'level complete': (236,196,63),
          'dark text': (119, 110, 101),
          'return': (246,94,59),
          'other': (188,172,159),
          'bg': (187, 173, 160)}

# game variables initialize
board_values = [[0 for _ in range(4)] for _ in range(4)]
game_over = False
spawn_new = True
init_count = 0
direction = ''
score = 0
high_score = 0
languages = {'english': {'score': 'Score:', 'high_score': 'High Score:', 'game_over': 'Game Over!', 'restart': 'Press Enter to Restart'},}
current_language = 'english'



# background music
mixer.music.load('musics/Wallpaper.wav')

#sound effect
game_over_sound = mixer.Sound('musics/gameover.wav')
mixer.music.play(-1)
win_sound = mixer.Sound('musics/level complete.wav')



# drawing the game screen
def draw_board():
    pygame.draw.rect(screen, colors['bg'], [0, 0, 400, 400], 0, 10)
    score_text = font.render(f"{languages[current_language]['score']} {score}", True, 'black')
    high_score_text = font.render(f"{languages[current_language]['high_score']} {high_score}", True, 'black')
    screen.blit(score_text, (10, 410))
    screen.blit(high_score_text, (10, 450))
    
    
    
# condtion for the game over
def game_over_condition(board):
    for i in range(4):
        for j in range(4):
            if board[i][j] == 0:
                return False
            if i < 3 and board[i][j] == board[i + 1][j]:
                return False
            if j < 3 and board[i][j] == board[i][j + 1]:
                return False
    return True
#drawing game over screen
def draw_over():
    pygame.draw.rect(screen, colors['other'], [0, 0, WIDTH, HEIGHT], 0)  
    game_over_text = font.render('GAME OVER', True, colors['light text'])
    restart_text = font.render('Press Enter to Restart', True, colors['light text'])
    text_rect = game_over_text.get_rect(center=(WIDTH // 2, HEIGHT // 2 - 30))
    restart_rect = restart_text.get_rect(center=(WIDTH // 2, HEIGHT // 2 + 30))
    screen.blit(game_over_text, text_rect)
    screen.blit(restart_text, restart_rect)



# take your turn based on direction + win
won = False
def take_turn(direc, board):
    global score, won # let score effect outside of function
    merged = [[False for _ in range(4)] for _ in range(4)]
    
    if direc == 'UP':
        for i in range(4):
            for j in range(4):
                shift = 0
                if i > 0:
                    for q in range(i):
                        if board[q][j] == 0:
                            shift += 1
                    if shift > 0:
                        board[i - shift][j] = board[i][j]
                        board[i][j] = 0
                    if board[i - shift - 1][j] == board[i - shift][j] and not merged[i - shift][j] \
                            and not merged[i - shift - 1][j]:
                        board[i - shift - 1][j] *= 2
                        score += board[i - shift - 1][j]
                        board[i - shift][j] = 0
                        merged[i - shift - 1][j] = True

    elif direc == 'DOWN':
        for i in range(3):
            for j in range(4):
                shift = 0
                for q in range(i + 1):
                    if board[3 - q][j] == 0:
                        shift += 1
                if shift > 0:
                    board[2 - i + shift][j] = board[2 - i][j]
                    board[2 - i][j] = 0
                if 3 - i + shift <= 3:
                    if board[2 - i + shift][j] == board[3 - i + shift][j] and not merged[3 - i + shift][j] \
                            and not merged[2 - i + shift][j]:
                        board[3 - i + shift][j] *= 2
                        score += board[3 - i + shift][j]
                        board[2 - i + shift][j] = 0
                        merged[3 - i + shift][j] = True

    elif direc == 'LEFT':
        for i in range(4):
            for j in range(4):
                shift = 0
                for q in range(j):
                    if board[i][q] == 0:
                        shift += 1
                if shift > 0:
                    board[i][j - shift] = board[i][j]
                    board[i][j] = 0
                if board[i][j - shift] == board[i][j - shift - 1] and not merged[i][j - shift - 1] \
                        and not merged[i][j - shift]:
                    board[i][j - shift - 1] *= 2
                    score += board[i][j - shift - 1]
                    board[i][j - shift] = 0
                    merged[i][j - shift - 1] = True

    elif direc == 'RIGHT':
        for i in range(4):
            for j in range(4):
                shift = 0
                for q in range(j):
                    if board[i][3 - q] == 0:
                        shift += 1
                if shift > 0:
                    board[i][3 - j + shift] = board[i][3 - j]
                    board[i][3 - j] = 0
                if 4 - j + shift <= 3:
                    if board[i][4 - j + shift] == board[i][3 - j + shift] and not merged[i][4 - j + shift] \
                            and not merged[i][3 - j + shift]:
                        board[i][4 - j + shift] *= 2
                        score += board[i][4 - j + shift]
                        board[i][3 - j + shift] = 0
                        merged[i][4 - j + shift] = True
    for row in board:
        if 2048 in row:
            won = True
            mixer.music.stop()  
            win_sound.play()
            break
    return board

# drawing winning screen
def draw_win():
    pygame.draw.rect(screen, colors['other'], [0, 0, WIDTH, HEIGHT], 0) 
    win_text = big_letter.render('Level Complete!', True, colors['level complete'])
    continue_text = font.render('Enter to Restart', True, colors['return'])
    text_rect = win_text.get_rect(center=(WIDTH // 2, HEIGHT // 2 - 30))
    continue_rect = continue_text.get_rect(center=(WIDTH // 2, HEIGHT // 2 + 30))
    screen.blit(win_text, text_rect)
    screen.blit(continue_text, continue_rect)




# creating random blocks
def new_pieces(board):
    count = 0
    full = False
    while any(0 in row for row in board) and count < 1:
        row = random.randint(0, 3)
        col = random.randint(0, 3)
        if board[row][col] == 0:
            count += 1
            if random.randint(1, 10) == 10:
                board[row][col] = 4
            else:
                board[row][col] = 2
    if count < 1:
        full = True
    return board, full
def draw_pieces(board):
    for i in range(4):
        for j in range(4):
            value = board[i][j]
            if value > 8:
                value_color = colors['light text']
            else:
                value_color = colors['dark text']
            if value <= 2048:
                color = colors[value]
            else:
                color = colors['other']
            pygame.draw.rect(screen, color, [j * 95 + 20, i * 95 + 20, 75, 75], 0, 5)
            if value > 0:
                value_len = len(str(value))
                font = pygame.font.Font('freesansbold.ttf', 48 - (5 * value_len))
                value_text = font.render(str(value), True, value_color)
                text_rect = value_text.get_rect(center=(j * 95 + 57, i * 95 + 57))
                screen.blit(value_text, text_rect)
                pygame.draw.rect(screen, 'black', [j * 95 + 20, i * 95 + 20, 75, 75], 2, 5)



# drawing the loading screen 
def draw_loading_screen():
    screen.fill(colors['bg'])
    loading_text = font.render('Choose Language:', True, colors['dark text'])
    english_text = font.render("English", True, colors["dark text"])
    screen.blit(loading_text, (WIDTH // 2 - 80, HEIGHT // 2 - 80))
    pygame.display.flip()
def loading_screen():
    loading = True
    loading_start_time = time.time()

    while loading:
        screen.fill(colors['bg'])  
        loading_font = pygame.font.Font('freesansbold.ttf', 32)
        loading_text = loading_font.render('Loading...', True, colors['light text'])
        loading_rect = loading_text.get_rect(center=(WIDTH // 2, HEIGHT // 2))
        screen.blit(loading_text, loading_rect)
        if time.time() - loading_start_time > 1:
            loading = False

        pygame.display.flip()
        pygame.time.Clock().tick(60)
        


# making menu screen
def menu_screen():
    while True:
        screen.fill((188,172,159))
        menu_font = pygame.font.Font('freesansbold.ttf', 32)
        title_font = pygame.font.Font('freesansbold.ttf', 40)
        
        title = title_font.render('2048 Game', True, (237,207,114))
        play_text = menu_font.render('Play Game', True, (59,57,55))
        quit_text = menu_font.render('Quit', True, (59,57,55))

        title_rect = title.get_rect(center=(WIDTH // 2, HEIGHT // 13))
        play_rect = play_text.get_rect(center=(WIDTH // 2, HEIGHT // 3))
        quit_rect = quit_text.get_rect(center=(WIDTH // 2, HEIGHT // 4 * 3))
        
        screen.blit(title,title_rect)
        screen.blit(play_text, play_rect)
        screen.blit(quit_text, quit_rect)

        for event in pygame.event.get():
            if event.type == pygame.QUIT:
                pygame.quit()
                sys.exit()
            elif event.type == pygame.MOUSEBUTTONDOWN:
                if play_rect.collidepoint(event.pos):
                    loading_screen()
                    return  
                elif quit_rect.collidepoint(event.pos):
                    pygame.quit()
                    sys.exit()  

        pygame.display.flip()
        pygame.time.Clock().tick(100)  



# call the menu screen function
menu_screen()


# main in game
game_over_sound_played = False
run = True
while run:
    timer.tick(fps)
    screen.fill('gray')
    draw_board()
    draw_pieces(board_values)
    score_text = font.render(f"{languages[current_language]['score']} {score}", True, 'black')
    # creating new random blocks
    if spawn_new or init_count < 2:
        board_values, game_over = new_pieces(board_values)
        spawn_new = False
        init_count += 1
    # movment
    if direction != '':
        board_values = take_turn(direction, board_values)
        direction = ''
        spawn_new = True
    # wining condtion
    if won:
        draw_win()
        for event in pygame.event.get():
            if event.type == pygame.QUIT:
                run = False
            elif event.type == pygame.KEYUP:
                # Reset the game 
                if event.key == pygame.K_RETURN:
                    board_values = [[0 for _ in range(4)] for _ in range(4)]
                    game_over = False
                    score = 0
                    spawn_new = True
                    init_count = 0
                    won = False  
                elif event.key == pygame.K_c:
                    won = False
    # game over condition
    if game_over_condition(board_values) and not game_over:
        game_over = True
        mixer.music.stop()
        if not game_over_sound_played:
            game_over_sound.play()
            game_over_sound_played = True
    

    for event in pygame.event.get():
        if event.type == pygame.QUIT:
            run = False
            
        if event.type == pygame.KEYUP:
            if event.key == pygame.K_UP:
                direction = 'UP'
            elif event.key == pygame.K_DOWN:
                direction = 'DOWN'
            elif event.key == pygame.K_LEFT:
                direction = 'LEFT'
            elif event.key == pygame.K_RIGHT:
                direction = 'RIGHT'
            
            #Just For Test Winning Condition is Working!!!
            elif event.key == pygame.K_c:  
                for i in range(4):
                    for j in range(4):
                        if board_values[i][j] > 0:
                            board_values[i][j] *= 2  
            # reseting value
            if game_over:
                draw_over()
                for event in pygame.event.get():
                    if event.type == pygame.QUIT:
                        run = False
                    elif event.type == pygame.KEYUP and event.key == pygame.K_RETURN:
                        # Reset the game state
                        board_values = [[0 for _ in range(4)] for _ in range(4)]
                        game_over = False
                        score = 0
                        spawn_new = True
                        init_count = 0
                        game_over_sound_played = False 
                        mixer.music.play(-1)
        if score > high_score:
            high_score = score
        if game_over:
            draw_over()

    pygame.display.flip()
    
pygame.quit()